﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="ExtendedCheckoutController.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2016
// </copyright>
// <summary>
//   Extended checkout controller class
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Controllers;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Commerce.Runtime.DataModel;
    using Microsoft.Dynamics.Commerce.Runtime.Client;
    using System.Collections.Generic;
    using CrtDataModel = Microsoft.Dynamics.Commerce.Runtime.DataModel;
    using System.Linq;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Mappers;
    
    /// <summary>
    /// Extended checkout controller class
    /// </summary>
    public class ExtendedCheckoutController : CheckoutController
    {
        private static readonly object SyncRoot = new object();
        private ShoppingCartController _shoppingCartControllerValue;

        /// <summary>
        /// Gets an instance of ShoppingCartController.
        /// </summary>
        protected override ShoppingCartController ShoppingCartController
        {
            get
            {
                if (this._shoppingCartControllerValue == null)
                {
                    lock (SyncRoot)
                    {
                        if (this._shoppingCartControllerValue == null)
                        {
                            this._shoppingCartControllerValue = new ExtendedShoppingCartController();
                            this._shoppingCartControllerValue.SetConfiguration(this.Configuration);
                        }
                    }
                }

                return this._shoppingCartControllerValue;
            }
        }

        /// <summary>
        /// Updates the shipping address.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="shippingAddress">The shipping address.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <param name="productValidator">The product validator.</param>
        /// <param name="searchEngine">The search engine.</param>
        /// <returns></returns>
        /// <exception cref="System.ArgumentNullException">cartId
        /// or
        /// shippingAddress</exception>
        /// <exception cref="System.InvalidOperationException">
        /// OrderManager can not be null when updating a shipping addres
        /// or
        /// Shopping cart can not be null when updating a shipping addres
        /// </exception>
        /// <exception cref="System.NullReferenceException">orderManager
        /// or
        /// cart</exception>
        public ShoppingCart UpdateOrderShippingAddress(string cartId, string customerId, Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models.Address shippingAddress, ShoppingCartDataLevel dataLevel, IProductValidator productValidator, ISearchEngine searchEngine)
        {
            if (string.IsNullOrWhiteSpace(cartId))
            {
                throw new ArgumentNullException("cartId");
            }

            if (shippingAddress == null)
            {
                throw new ArgumentNullException("shippingAddress");
            }

            OrderManager orderManager = OrderManager.Create(CrtUtilities.GetCommerceRuntime());
            if (orderManager == null)
            {
                throw new InvalidOperationException("OrderManager can not be null when updating a shipping addres");
            }

            Cart cart = orderManager.GetCart(cartId, customerId, CalculationModes.All);
            if (cart == null)
            {
                throw new InvalidOperationException("Shopping cart can not be null when updating a shipping addres");
            }

            var deliveryAddress = ExtendedAddressMapper.ConvertToDataModel(shippingAddress);
            cart = orderManager.UpdateCartShippingAddress(cartId, customerId, deliveryAddress, cart.DeliveryMode, (CalculationModes)sbyte.MaxValue);
            var shoppingCartMapper = new ExtendedShoppingCartMapper(this.Configuration);
            var cartModel = shoppingCartMapper.ConvertToViewModel(cart, dataLevel, productValidator, searchEngine);
            return cartModel;
        }

        /// <summary>
        /// Updates the shipping method.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="selectedDeliveryOption">The selected delivery option.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <param name="productValidator">The product validator.</param>
        /// <param name="searchEngine">The search engine.</param>
        /// <returns></returns>
        /// <exception cref="System.ArgumentNullException">cartId
        /// or
        /// shippingOptions</exception>
        /// <exception cref="System.InvalidOperationException">OrderManager can not be null when updating a shipping method
        /// or
        /// Cart can not be null when updating a shipping method</exception>
        /// <exception cref="System.NullReferenceException">orderManager
        /// or
        /// cart</exception>
        public ShoppingCart UpdateOrderShippingMethod(string cartId, string customerId, SelectedDeliveryOption selectedDeliveryOption, ShoppingCartDataLevel dataLevel, IProductValidator productValidator, ISearchEngine searchEngine)
        {
            if (string.IsNullOrWhiteSpace(cartId))
            {
                throw new ArgumentNullException("cartId");
            }

            if (selectedDeliveryOption == null)
            {
                throw new ArgumentNullException("selectedDeliveryOption");
            }

            OrderManager orderManager = OrderManager.Create(CrtUtilities.GetCommerceRuntime());
            if (orderManager == null)
            {
                throw new InvalidOperationException("OrderManager can not be null when updating a shipping method");
            }

            Cart cart = orderManager.GetCart(cartId, customerId, CalculationModes.All);
            if (cart == null)
            {
                throw new InvalidOperationException("Cart can not be null when updating a shipping method");
            }

            string warehouseId = string.Empty;
            var deliveryAddress = cart.ShippingAddress ?? new Microsoft.Dynamics.Commerce.Runtime.DataModel.Address();
            cart = orderManager.UpdateCartShippingAddress(cartId, customerId, deliveryAddress, selectedDeliveryOption.DeliveryModeId, (CalculationModes)sbyte.MaxValue, warehouseId);
            var shoppingCartMapper = new ExtendedShoppingCartMapper(this.Configuration);
            var cartModel = shoppingCartMapper.ConvertToViewModel(cart, dataLevel, productValidator, searchEngine);
            return cartModel;
        }

        /// <summary>
        /// Deletes the carts.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="cartIds">The cart ids.</param>
        public void DeleteCarts(string customerId, IEnumerable<string> cartIds)
        {
            if (cartIds == null)
            {
                throw new ArgumentNullException("cartIds");
            }

            OrderManager orderManager = OrderManager.Create(CrtUtilities.GetCommerceRuntime());
            if (orderManager == null)
            {
                throw new InvalidOperationException("OrderManager can not be null when updating a shipping method");
            }

            orderManager.DeleteCarts(customerId, cartIds);
        }

        /// <summary>
        /// Commits the selected delivery option to the cart when entire order is being 'delivered' as a single entity.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="selectedDeliveryOption">The selected delivery option.</param>
        /// <param name="productValidator">The product validator.</param>
        /// <param name="searchEngine">The search engine.</param>
        /// <param name="dataLevel">The shopping cart data level.</param>
        /// <returns>
        /// The updated shopping cart.
        /// </returns>
        /// <exception cref="System.ArgumentNullException">Thrown when the shoppingCartId or shippingOptions is null or empty.</exception>
        public override ShoppingCart SetOrderDeliveryOption(string shoppingCartId, string customerId, SelectedDeliveryOption selectedDeliveryOption, IProductValidator productValidator, ISearchEngine searchEngine, ShoppingCartDataLevel dataLevel)
        {
            if (string.IsNullOrWhiteSpace(shoppingCartId))
            {
                throw new ArgumentNullException("shoppingCartId");
            }

            if (selectedDeliveryOption == null)
            {
                throw new ArgumentNullException("selectedDeliveryOption");
            }

            this.ValidateSelectedDeliveryOption(selectedDeliveryOption);

            OrderManager manager = OrderManager.Create(CrtUtilities.GetCommerceRuntime());

            CrtDataModel.Address address = GetDeliveryAddress(selectedDeliveryOption);
            Cart cart;
            string warehouseId = string.Empty;

            CrtDataModel.DeliveryPreferenceType selectedDeliveryPreferenceType = GetDeliveryPreferenceType(selectedDeliveryOption);

            switch (selectedDeliveryPreferenceType)
            {
                case CrtDataModel.DeliveryPreferenceType.PickupFromStore:
                    // CHANGING CALCULATION MODES TO NONE DUE TO MSFT BUG 7364
                    cart = manager.GetCart(shoppingCartId, customerId, CalculationModes.None);
                    cart.ShippingAddress = address;
                    cart.DeliveryMode = selectedDeliveryOption.DeliveryModeId;
                    cart = manager.CreateOrUpdateCart(cart);

                    // WHEN DELIVERY TO STORE, CART LINE STORE NUMBER MUST BE PROVIDED
                    foreach (CartLine cartLine in cart.CartLines)
                    {
                        cartLine.StoreNumber = selectedDeliveryOption.StoreAddress.StoreId;
                    }
                    cart = manager.UpdateCartLines(cart.Id, cart.CartLines);
                    break;

                case CrtDataModel.DeliveryPreferenceType.ElectronicDelivery:
                    // CHECKING ADDRESS ISN'T NULL DUE TO MSFT BUG 7695
                    if (address == null)
                    {
                        address = new CrtDataModel.Address();
                    }
                    
                    cart = manager.UpdateCartShippingAddress(
                        shoppingCartId,
                        customerId,
                        address,
                        selectedDeliveryOption.DeliveryModeId,
                        CalculationModes.All,
                        warehouseId,
                        selectedDeliveryOption.ElectronicDeliveryEmail,
                        selectedDeliveryOption.ElectronicDeliveryEmailContent);
                    break;

                default:
                    cart = manager.UpdateCartShippingAddress(
                        shoppingCartId,
                        customerId,
                        address,
                        selectedDeliveryOption.DeliveryModeId,
                        CalculationModes.All,
                        warehouseId);
                    break;
            }

            ShoppingCart shoppingCart;
            if (dataLevel == ShoppingCartDataLevel.All)
            {
                shoppingCart = ((ExtendedShoppingCartController)this.ShoppingCartController).AddKitDetailsToShoppingCart(cart, productValidator, searchEngine);
            }
            else
            {
                var mapper = new ExtendedShoppingCartMapper(this.Configuration);
                shoppingCart = mapper.ConvertToViewModel(cart, dataLevel, productValidator, searchEngine);
            }

            return shoppingCart;
        }
        
        /// <summary>
        /// Updates cart lines shipping addresses.
        /// </summary>
        /// <param name="selectedLineDeliveryOptions">The selected shipping options.</param>   
        /// <param name="lines">Cartlines to be updated.</param>
        protected override void UpdateCartLineShippingAddresses(IEnumerable<SelectedLineDeliveryOption> selectedLineDeliveryOptions, IEnumerable<CartLine> lines)
        {
            if (selectedLineDeliveryOptions == null)
            {
                throw new ArgumentNullException("selectedLineDeliveryOptions");
            }

            foreach (SelectedLineDeliveryOption lineDeliveryOption in selectedLineDeliveryOptions)
            {
                ValidateSelectedDeliveryOption(lineDeliveryOption);
                // CHECKING ADDRESS ISN'T NULL DUE TO MSFT BUG 7695
                CrtDataModel.Address shippingAddress = GetDeliveryAddress(lineDeliveryOption) ?? new CrtDataModel.Address();

                CartLine cartLine = lines.Single(l => l.LineId == lineDeliveryOption.LineId);
                cartLine.LineData.ShippingAddress = shippingAddress;
                cartLine.LineData.DeliveryMode = lineDeliveryOption.DeliveryModeId;

                CrtDataModel.DeliveryPreferenceType selectedDeliveryPreferenceType = GetDeliveryPreferenceType(lineDeliveryOption);
                if (selectedDeliveryPreferenceType == CrtDataModel.DeliveryPreferenceType.PickupFromStore)
                {
                    cartLine.LineData.StoreNumber = lineDeliveryOption.StoreAddress.StoreId;
                }

                if (selectedDeliveryPreferenceType == CrtDataModel.DeliveryPreferenceType.ElectronicDelivery)
                {
                    cartLine.LineData.ElectronicDeliveryEmail = lineDeliveryOption.ElectronicDeliveryEmail;
                    cartLine.LineData.ElectronicDeliveryEmailContent = lineDeliveryOption.ElectronicDeliveryEmailContent;
                }
            }
        }

        /// <summary>
        /// Gets the channel configuration.
        /// </summary>
        /// <returns>The channel configuration</returns>
        public virtual Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models.ChannelConfiguration GetChannelConfiguration()
        {
            var channelController = new ChannelController();
            channelController.SetConfiguration(this.Configuration);
            return channelController.GetChannelConfiguration();
        }
    }
}