﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="CheckoutService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2016
// </copyright>
// <summary>
//   The checkout dynamics service.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System.Linq;
    using Microsoft.Dynamics.Retail.Diagnostics;
    using System;
    using System.Collections.Generic;
    using System.ServiceModel.Activation;
    using System.ServiceModel.Web;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Controllers;
    using System.Diagnostics.CodeAnalysis;

    /// <summary>
    /// Service for checkout.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class CheckoutService : CheckoutServiceBase, IExtendedCheckoutService
    {
        private static readonly ISearchEngine searchEngine = new SearchEngine();
        private static readonly IProductValidator productValidator = new ProductValidator();
        private static readonly ServiceUtilityBase serviceProxy = new ServiceUtility();

        /// <summary>
        /// Gets Demo implementation of ISearchEngine.
        /// </summary>
        protected override ISearchEngine SearchEngine
        {
            get { return searchEngine; }
        }

        /// <summary>
        /// Gets Demo implementation of IProductValidator.
        /// </summary>
        protected override IProductValidator ProductValidator
        {
            get { return productValidator; }
        }

        /// <summary>
        /// Gets Demo implementation of ServiceUtilityBase.
        /// </summary>
        protected override ServiceUtilityBase ServiceUtility
        {
            get { return serviceProxy; }
        }

        /// <summary>
        /// Gets Demo implementation of IConfiguration.
        /// </summary>
        protected override IConfiguration Configuration
        {
            get { return new ServiceConfiguration(); }
        }
        
        #region EXTENSIONS
        
        /// <summary>
        /// Creates the controller.
        /// </summary>
        /// <returns>This controller</returns>
        protected override CheckoutController CreateController()
        {
            return Activator.CreateInstance<ExtendedCheckoutController>();
        }

        /// <summary>
        /// Updates the shipping address.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="shippingAddress">The shipping address.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <returns>
        /// A sbopping cart
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse UpdateOrderShippingAddress(string cartId, string customerId, Address shippingAddress, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
            }

            if (shippingAddress == null)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("shippingAddress")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = ((ExtendedCheckoutController)this.Controller).UpdateOrderShippingAddress(cartId, customerId, shippingAddress, dataLevel, this.ProductValidator, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Updates the shipping method.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="selectedDeliveryOption">The selected delivery option.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <returns>
        /// A sbopping cart
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse UpdateOrderShippingMethod(string cartId, string customerId, SelectedDeliveryOption selectedDeliveryOption, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
            }

            if (selectedDeliveryOption == null)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("selectedDeliveryOption")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = ((ExtendedCheckoutController)this.Controller).UpdateOrderShippingMethod(cartId, customerId, selectedDeliveryOption, dataLevel, this.ProductValidator, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Gets all delivery methods extended.
        /// </summary>
        /// <returns>
        /// A list of available delivery options
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ExtendedDeliveryOptionsResponse GetDeliveryOptionsInfoExtended()
        {
            var response = new ExtendedDeliveryOptionsResponse();

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.SetDeliveryOptions(this.Controller.GetDeliveryOptionsInfo());

                var channelConfig = ((ExtendedCheckoutController)this.Controller).GetChannelConfiguration();
                response.EmailDeliveryModeCode = channelConfig.EmailDeliveryModeCode;
                response.PickupDeliveryModeCode = channelConfig.PickupDeliveryModeCode;
            });
        
            return response;
        }

        /// <summary>
        /// Gets delivery preferences applicable to the current checkout cart.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <returns>
        /// The delivery preference response object.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual DeliveryPreferenceResponse GetDeliveryPreferencesExtended(string cartId)
        {
            var response = new DeliveryPreferenceResponse();
            if (string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.CartDeliveryPreferences = this.Controller.GetDeliveryPreferences(cartId);
            });

            return response;
        }

        /// <summary>
        /// Gets the delivery methods extended.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="shipToAddress">The ship to address.</param>
        /// <returns>
        /// A list of delivery options
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ExtendedDeliveryOptionsResponse GetOrderDeliveryOptionsForShippingExtended(string cartId, string customerId, Address shipToAddress)
        {
            var response = new ExtendedDeliveryOptionsResponse();
            if (string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
            }

            if (shipToAddress == null)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("shipToAddress")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.SetDeliveryOptions(this.Controller.GetOrderDeliveryOptionsForShipping(cartId, customerId, shipToAddress));
               
                var channelConfig = ((ExtendedCheckoutController)this.Controller).GetChannelConfiguration();
                response.EmailDeliveryModeCode = channelConfig.EmailDeliveryModeCode;
                response.PickupDeliveryModeCode = channelConfig.PickupDeliveryModeCode;
            });
            
            return response;
        }

        /// <summary>
        /// Gets the delivery methods per item extended.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="selectedLineShippingInfo">The selected line shipping information.</param>
        /// <returns>
        /// A list of delivery options
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ExtendedDeliveryOptionsResponse GetLineDeliveryOptionsForShippingExtended(string cartId, string customerId, IEnumerable<SelectedLineShippingInfo> selectedLineShippingInfo)
        {
            var response = new ExtendedDeliveryOptionsResponse();
            if (string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
            }

            var selectedLineShippingInfos = selectedLineShippingInfo as SelectedLineShippingInfo[] ?? selectedLineShippingInfo.ToArray();
            if (selectedLineShippingInfo == null || !selectedLineShippingInfos.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("selectedLineShippingInfo")));
            }

            if (response.HasErrors)
            {
                return response;
            }
            
            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.SetLineDeliveryOptions(this.Controller.GetLineDeliveryOptionsForShipping(cartId, customerId, selectedLineShippingInfos));

                var channelConfig = ((ExtendedCheckoutController)this.Controller).GetChannelConfiguration();
                response.EmailDeliveryModeCode = channelConfig.EmailDeliveryModeCode;
                response.PickupDeliveryModeCode = channelConfig.PickupDeliveryModeCode;
            });
            
            return response;
        }

        /// <summary>
        /// Sets the delivery method extended.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="selectedDeliveryOption">The selected delivery option.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <returns>
        /// Set order's delivery option
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse  SetOrderDeliveryOptionExtended(string cartId, string customerId, SelectedDeliveryOption selectedDeliveryOption, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
            }

            if (selectedDeliveryOption == null)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("selectedDeliveryOption")));
            }

            if (response.HasErrors)
            {
                return response;
            }
            
            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = this.Controller.SetOrderDeliveryOption(cartId, customerId, selectedDeliveryOption, this.ProductValidator, this.SearchEngine, dataLevel);
            });

            return response;
        }

        /// <summary>
        /// Sets the delivery method per item extended.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="selectedLineDeliveryOptions">The selected line delivery options.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <returns>
        /// Set lines' delivery options
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse SetLineDeliveryOptionsExtended(string cartId, string customerId, IEnumerable<SelectedLineDeliveryOption> selectedLineDeliveryOptions, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
            }

            var lineDeliveryOptions = selectedLineDeliveryOptions as SelectedLineDeliveryOption[] ?? selectedLineDeliveryOptions.ToArray();
            if (selectedLineDeliveryOptions == null || !lineDeliveryOptions.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("selectedLineDeliveryOptions")));
            }

            if (response.HasErrors)
            {
                return response;
            }
            
            this.ExecuteAfterRequestValidation(response, () =>
            {
                var shoppingCart = this.Controller.SetLineDeliveryOptions(cartId, customerId, lineDeliveryOptions, this.ProductValidator, this.SearchEngine, dataLevel);
                // CLEARING OUT SHIPPING INFO AT THE ORDER LEVEL WHEN SETTING SHIPPING INFO AT LINE LEVEL
                // SO WE CAN DIFFERENTIATE BETWEEN ORDER LEVEL SHIPPING AND ITEM LEVEL SHIPPING
                shoppingCart.ShippingAddress = null;
                shoppingCart.DeliveryModeId = null;
                shoppingCart.DeliveryModeDescription = null;
                response.ShoppingCart = shoppingCart;
            });

            return response;
        }
        
        /// <summary>
        /// Creates the order extended.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="tenderDataLine">The tender data line.</param>
        /// <param name="emailAddress">The email address.</param>
        /// <returns></returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ExtendedCreateSalesOrderResponse CreateOrderExtended(string cartId, string customerId, IEnumerable<TenderDataLine> tenderDataLine, string emailAddress)
        {
            var response = new ExtendedCreateSalesOrderResponse();
            if (string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
            } 
            
            if (string.IsNullOrEmpty(emailAddress))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("emailAddress")));
            }

            var tenderDataLines = tenderDataLine as TenderDataLine[] ?? tenderDataLine.ToArray();
            if (tenderDataLine == null || !tenderDataLines.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("tenderDataLine")));
            }

            if (response.HasErrors)
            {
                return response;
            }
            
            this.ExecuteAfterRequestValidation(response, () =>
            {
                IEnumerable<string> linesToDelete = this.Controller.GetShoppingCart(cartId, customerId, ShoppingCartDataLevel.Minimal, this.ProductValidator, this.SearchEngine).Items.Select(i => i.LineId);
                var orderNumber = this.Controller.CreateOrder(cartId, customerId, tenderDataLines, emailAddress, emailAddress);
                if (!string.IsNullOrEmpty(orderNumber))
                {
                    var orderController = Activator.CreateInstance<OrderController>();
                    orderController.SetConfiguration(this.Configuration);
                    response.SalesOrder = orderController.GetCommittedSalesOrderByConfirmationId(orderNumber, true, this.SearchEngine);
                    response.OrderNumber = orderNumber;
                }

                // REMOVE ITEMS FROM ANONYMOUS USER'S CART
                try
                {
                    this.Controller.RemoveItems(cartId, customerId, linesToDelete, ShoppingCartDataLevel.Minimal, this.ProductValidator, this.SearchEngine);
                }
                catch (Exception e)
                {
                    NetTracer.Error(e, "Failed to delete shopping cart after order creation.", new object[0]);
                }
            });

            return response;
        }
        
        #endregion

        /// <summary>
        /// Gets details of all the delivery options available for the current legal entity.
        /// </summary>
        /// <returns>
        /// The delivery options response object.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override DeliveryOptionsResponse GetDeliveryOptionsInfo()
        {
            return base.GetDeliveryOptionsInfo();
        }

        /// <summary>
        /// Gets delivery preferences applicable to the current checkout cart.
        /// </summary>
        /// <returns>
        /// The delivery preference response object.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override DeliveryPreferenceResponse GetDeliveryPreferences()
        {
            return base.GetDeliveryPreferences();
        }

        /// <summary>
        /// Gets the applicable delivery options when the user wants to 'ship' the entire order as a single entity.
        /// </summary>
        /// <param name="shipToAddress">The 'ship to' address.</param>
        /// <returns>
        /// The service response containing the available delivery options.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override DeliveryOptionsResponse GetOrderDeliveryOptionsForShipping(Address shipToAddress)
        {
            DeliveryOptionsResponse response = new DeliveryOptionsResponse();
            if (shipToAddress == null)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("shipToAddress")));
            }

            return response.HasErrors ? response : base.GetOrderDeliveryOptionsForShipping(shipToAddress);
        }

        /// <summary>
        /// Gets the delivery options applicable per line when the user wants the items in the cart 'shipped' to them individually.
        /// </summary>
        /// <param name="selectedLineShippingInfo">The shipping information for the selected lines.</param>
        /// <returns>
        /// The service response containing the available delivery options for the specified lines.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override DeliveryOptionsResponse GetLineDeliveryOptionsForShipping(IEnumerable<SelectedLineShippingInfo> selectedLineShippingInfo)
        {
            DeliveryOptionsResponse response = new DeliveryOptionsResponse();
            var selectedLineShippingInfos = selectedLineShippingInfo as SelectedLineShippingInfo[] ?? selectedLineShippingInfo.ToArray();
            if (selectedLineShippingInfo == null || !selectedLineShippingInfos.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("selectedLineShippingInfo")));
            }

            return response.HasErrors ? response : base.GetLineDeliveryOptionsForShipping(selectedLineShippingInfos);
        }

        /// <summary>
        /// Commits the selected delivery option to the cart when entire order is being 'delivered' as a single entity.
        /// </summary>
        /// <param name="selectedDeliveryOption">The selected delivery option.</param>
        /// <param name="dataLevel">The shopping cart data level.</param>
        /// <returns>
        /// The updated shopping cart.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse SetOrderDeliveryOption(SelectedDeliveryOption selectedDeliveryOption, ShoppingCartDataLevel dataLevel)
        {
            ShoppingCartResponse response = new ShoppingCartResponse();
            if (selectedDeliveryOption == null)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("selectedDeliveryOption")));
            }

            return response.HasErrors ? response : base.SetOrderDeliveryOption(selectedDeliveryOption, dataLevel);
        }

        /// <summary>
        /// Commits the selected delivery options per line when the sales line in the order are being 'delivered' individually.
        /// </summary>
        /// <param name="selectedLineDeliveryOptions">The line delivery options.</param>
        /// <param name="dataLevel">The shopping cart data level.</param>
        /// <returns>
        /// The updated shopping cart.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse SetLineDeliveryOptions(IEnumerable<SelectedLineDeliveryOption> selectedLineDeliveryOptions, ShoppingCartDataLevel dataLevel)
        {
            ShoppingCartResponse response = new ShoppingCartResponse();
            var lineDeliveryOptions = selectedLineDeliveryOptions as SelectedLineDeliveryOption[] ?? selectedLineDeliveryOptions.ToArray();
            if (selectedLineDeliveryOptions == null || !lineDeliveryOptions.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("selectedLineDeliveryOptions")));
            }

            return response.HasErrors ? response : base.SetLineDeliveryOptions(lineDeliveryOptions, dataLevel);
        }

        /// <summary>
        /// Processes a sales order.
        /// </summary>
        /// <param name="tenderDataLine">Tender lines data containing payments for the order.</param>
        /// <param name="emailAddress">The email address.</param>
        /// <returns>
        /// The updated shopping cart item collection.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override CreateSalesOrderResponse CreateOrder(IEnumerable<TenderDataLine> tenderDataLine, string emailAddress)
        {
            CreateSalesOrderResponse response = new CreateSalesOrderResponse();
            if (tenderDataLine == null)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("tenderDataLine")));
            }

            if (string.IsNullOrEmpty(emailAddress))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("emailAddress")));
            }

            return response.HasErrors ? response : base.CreateOrder(tenderDataLine, emailAddress);
        }

        /// <summary>
        /// Gets the payment card types.
        /// </summary>
        /// <returns>
        /// The valid card types accepted for payment.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override PaymentCardTypesResponse GetPaymentCardTypes()
        {
            return base.GetPaymentCardTypes();
        }

        /// <summary>
        /// Gets the gift card balance.
        /// </summary>
        /// <returns>
        /// The gift card balance response.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override GiftCardResponse GetGiftCardInformation(string giftCardId)
        {
            var response = new GiftCardResponse();
            if (string.IsNullOrEmpty(giftCardId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("giftCardId")));
            }

            return response.HasErrors ? response : base.GetGiftCardInformation(giftCardId);
        }

        /// <summary>
        /// Gets boolean value indicating whether session is authenticated or not.
        /// </summary>
        /// <returns>True if session is authenticated.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override BooleanResponse IsAuthenticatedSession()
        {
            return base.IsAuthenticatedSession();
        }

        /// <summary>
        /// Gets customer email address.
        /// </summary>
        /// <returns>Email response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override StringResponse GetUserEmail()
        {
            return base.GetUserEmail();
        }

        /// <summary>
        /// Executes the given service action and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteAfterRequestValidation(ServiceResponse response, Action action)
        {
            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            var serviceUtility = ServiceUtility as ServiceUtility;
            if (serviceUtility != null)
            {
                serviceUtility.SetSslCertPolicy();
            }

            Utilities.ExecuteAndHandleExceptions(response, action);
        }

        /// <summary>
        /// Executes handler and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void Execute(ServiceResponse response, Action action)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Checks if user is authenticated and assosiated then calls <see cref="ExecuteAfterRequestValidation" /> with new execution handler.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void ExecuteForAuthorizedUser(ServiceResponse response, Action<string> action)
        {
            throw new NotImplementedException();
        }
    }
}
