﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="LoyaltyService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2016
// </copyright>
// <summary>
//   The loyalty dynamics service.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System;
    using System.ServiceModel.Activation;
    using System.ServiceModel.Web;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;

    /// <summary>
    /// The loyalty dynamics service.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class LoyaltyService : LoyaltyServiceBase, IExtendedLoyaltyService
    {
        private static readonly IProductValidator productValidator = new ProductValidator();
        private static readonly ISearchEngine searchEngine = new SearchEngine();
        private static readonly ServiceUtilityBase serviceProxy = new ServiceUtility();

        /// <summary>
        /// Gets Demo implementation of ISearchEngine.
        /// </summary>
        protected override ISearchEngine SearchEngine
        {
            get { return searchEngine; }
        }

        /// <summary>
        /// Gets Demo implementation of IProductValidator.
        /// </summary>
        protected override IProductValidator ProductValidator
        {
            get { return productValidator; }
        }

        /// <summary>
        /// Gets Demo implementation of ServiceUtilityBase.
        /// </summary>
        protected override ServiceUtilityBase ServiceUtility
        {
            get { return serviceProxy; }
        }

        /// <summary>
        /// Gets Demo implementation of IConfiguration.
        /// </summary>
        protected override IConfiguration Configuration
        {
            get { return new ServiceConfiguration(); }
        }

        #region EXTENSIONS

        /// <summary>
        /// Generates the loyalty card identifier.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <returns>
        /// A shopping cart response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse GenerateLoyaltyCardIdExtended(string cartId, string customerId, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(customerId) && string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = this.Controller.GenerateLoyaltyCardId(cartId, customerId, dataLevel, this.ProductValidator, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Gets the loyalty cards.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <returns>
        /// A loyalty cards response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual LoyaltyCardsResponse GetLoyaltyCardsExtended(string customerId)
        {
            var response = new LoyaltyCardsResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.LoyaltyCards = this.Controller.GetLoyaltyCards(customerId);
            });

            return response;
        }

        /// <summary>
        /// Gets the status of a loyalty card given it's id
        /// </summary>
        /// 
        /// <param name="loyaltyCardNumber">The loyalty card Id</param>
        /// <returns>A loyaltyCardResponse object</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual LoyaltyCardsResponse GetLoyaltyCardStatusExtended(string loyaltyCardNumber)
        {
            var response = new LoyaltyCardsResponse();
            if (string.IsNullOrEmpty(loyaltyCardNumber))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("loyaltyCardNumber")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                var loyaltyCard = this.Controller.GetLoyaltyCardStatus(loyaltyCardNumber);
                response.LoyaltyCards = new List<LoyaltyCard> { loyaltyCard };
            });

            return response;
        }

        /// <summary>
        /// Gets all loyalty cards status.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <returns>
        /// A loyalty cards response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual LoyaltyCardsResponse GetAllLoyaltyCardsStatusExtended(string customerId)
        {
            var response = new LoyaltyCardsResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.LoyaltyCards = this.Controller.GetAllLoyaltyCardsStatus(customerId);
            });

            return response;
        }

        /// <summary>
        /// Updates the loyalty card identifier.
        /// </summary>
        /// <param name="cartId">The cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="loyaltyCardId">The loyalty card identifier.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <returns>
        /// A shopping cart response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse UpdateLoyaltyCardIdExtended(string cartId, string customerId, string loyaltyCardId, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(cartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("cartId")));
            }

            if (string.IsNullOrEmpty(loyaltyCardId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("loyaltyCardId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = this.Controller.UpdateLoyaltyCardId(cartId, loyaltyCardId, customerId, dataLevel, this.ProductValidator, this.SearchEngine);
            });

            return response;
        }

        #endregion

        /// <summary>
        /// Issues a new loyalty card id for the customer.
        /// </summary>
        /// <param name="isCheckoutSession">Indicates whether this is in context of a checkout session.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The shopping cart.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse GenerateLoyaltyCardId(bool isCheckoutSession, ShoppingCartDataLevel dataLevel)
        {
            return base.GenerateLoyaltyCardId(isCheckoutSession, dataLevel);
        }

        /// <summary>
        /// Gets a read only collection of all loyalty card numbers
        /// </summary>
        /// 
        /// <returns>A loyaltyCardsResponse object</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services.LoyaltyCardsResponse GetLoyaltyCards()
        {
            return base.GetLoyaltyCards();
        }

        /// <summary>
        /// Gets the status of a loyalty card given it's id
        /// </summary>
        /// 
        /// <param name="loyaltyCardNumber">The loyalty card Id</param>
        /// <returns>A loyaltyCardResponse object</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override LoyaltyCardResponse GetLoyaltyCardStatus(string loyaltyCardNumber)
        {
            var response = new LoyaltyCardResponse();
            if (string.IsNullOrEmpty(loyaltyCardNumber))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("loyaltyCardNumber")));
            }

            return response.HasErrors ? response : base.GetLoyaltyCardStatus(loyaltyCardNumber);
        }

        /// <summary>
        /// Gets a read only collection of all loyalty card with their status
        /// </summary>
        /// 
        /// <returns>A loyaltyCardsResponse object</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services.LoyaltyCardsResponse GetAllLoyaltyCardsStatus()
        {
            return base.GetAllLoyaltyCardsStatus();
        }

        /// <summary>
        /// Gets a stream with all the transaction data specific to a a loyalty card number for a given points category
        /// </summary>
        /// 
        /// <param name="loyaltyCardNumber">The loyalty card Id</param>
        /// <param name="rewardPointId">The reward points Id</param>
        /// <param name="topRows">Number of entries that should appear per page</param>
        /// <returns>A LoyaltyCardTransactionsResponse object</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override LoyaltyCardTransactionsResponse GetLoyaltyCardTransactions(string loyaltyCardNumber, string rewardPointId, int topRows)
        {
            var response = new LoyaltyCardTransactionsResponse();
            if (string.IsNullOrEmpty(loyaltyCardNumber))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("loyaltyCardNumber")));
            }

            if (string.IsNullOrEmpty(rewardPointId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("rewardPointId")));
            }

            return response.HasErrors ? response : base.GetLoyaltyCardTransactions(loyaltyCardNumber, rewardPointId, topRows);
        }

        /// <summary>
        /// Updates the loyalty card id on the shopping cart.
        /// </summary>
        /// <param name="isCheckoutSession">Indicates whether this is in context of a checkout session.</param>
        /// <param name="loyaltyCardId">The loyalty card id (or empty string) to set on the cart.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The shopping cart.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse UpdateLoyaltyCardId(bool isCheckoutSession, string loyaltyCardId, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(loyaltyCardId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("loyaltyCardId")));
            }

            return response.HasErrors ? response : base.UpdateLoyaltyCardId(isCheckoutSession, loyaltyCardId, dataLevel);
        }

        /// <summary>
        /// Executes the given service action and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteAfterRequestValidation(ServiceResponse response, Action action)
        {
            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            var serviceUtility = ServiceUtility as ServiceUtility;
            if (serviceUtility != null)
            {
                serviceUtility.SetSslCertPolicy();
            }

            Utilities.ExecuteAndHandleExceptions(response, action);
        }

        /// <summary>
        /// Executes handler and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void Execute(ServiceResponse response, Action action)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Checks if user is authenticated and assosiated then calls <see cref="ExecuteAfterRequestValidation" /> with new execution handler.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void ExecuteForAuthorizedUser(ServiceResponse response, Action<string> action)
        {
            throw new NotImplementedException();
        }
    }
}
