﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="OrderService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2016
// </copyright>
// <summary>
//   The order dynamics service.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using System;
    using System.ServiceModel.Activation;
    using System.ServiceModel.Web;
    using System.Diagnostics.CodeAnalysis;

    /// <summary>
    /// The order dynamics service.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class OrderService : OrderServiceBase, IExtendedOrderService
    {
        private static readonly IProductValidator productValidator = new ProductValidator();
        private static readonly ISearchEngine searchEngine = new SearchEngine();
        private static readonly ServiceUtilityBase serviceProxy = new ServiceUtility();

        /// <summary>
        /// Gets Demo implementation of ISearchEngine.
        /// </summary>
        protected override ISearchEngine SearchEngine
        {
            get { return searchEngine; }
        }

        /// <summary>
        /// Gets Demo implementation of IProductValidator.
        /// </summary>
        protected override IProductValidator ProductValidator
        {
            get { return productValidator; }
        }

        /// <summary>
        /// Gets Demo implementation of ServiceUtilityBase.
        /// </summary>
        protected override ServiceUtilityBase ServiceUtility
        {
            get { return serviceProxy; }
        }

        /// <summary>
        /// Gets Demo implementation of IConfiguration.
        /// </summary>
        protected override IConfiguration Configuration
        {
            get { return new ServiceConfiguration(); }
        }

        #region EXTENSIONS

        /// <summary>
        /// Gets the sales orders by customer.
        /// </summary>
        /// <returns>The sales order collection.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual SalesOrderCollectionResponse GetSalesOrdersByCustomerExtended(string accountNumber, bool includeSalesLines)
        {
            var response = new SalesOrderCollectionResponse();
            if (string.IsNullOrEmpty(accountNumber))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("accountNumber")));
            }
            
            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.SalesOrders = this.Controller.GetSalesOrdersByCustomer(accountNumber, includeSalesLines, this.SearchEngine);
            });
            
            return response;
        }

        /// <summary>
        /// Gets the committed sales order.
        /// </summary>
        /// <param name="salesId">The sales identifier.</param>
        /// <param name="includeSalesLines">if set to <c>true</c> [include sales lines].</param>
        /// <returns>
        /// The commited sales order.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual SalesOrderResponse GetCommittedSalesOrderExtended(string salesId, bool includeSalesLines)
        {
            var response = new SalesOrderResponse();
            if (string.IsNullOrEmpty(salesId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("salesId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.SalesOrder = this.Controller.GetCommittedSalesOrder(salesId, includeSalesLines, this.SearchEngine);
            });
            
            return response;
        }

        /// <summary>
        /// Gets the committed sales order by confirmation identifier.
        /// </summary>
        /// <param name="confirmationId">The confirmation identifier.</param>
        /// <param name="includeSalesLines">if set to <c>true</c> [include sales lines].</param>
        /// <returns>
        /// The sales order.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual SalesOrderResponse GetCommittedSalesOrderByConfirmationIdExtended(string confirmationId, bool includeSalesLines)
        {
            var response = new SalesOrderResponse();
            if (string.IsNullOrEmpty(confirmationId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("confirmationId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.SalesOrder = this.Controller.GetCommittedSalesOrderByConfirmationId(confirmationId, includeSalesLines, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Gets the pending sales order.
        /// </summary>
        /// <param name="transactionId">The transaction identifier.</param>
        /// <param name="includeSalesLines">if set to <c>true</c> [include sales lines].</param>
        /// <returns>
        /// The sales order.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual SalesOrderResponse GetPendingSalesOrderExtended(string transactionId, bool includeSalesLines)
        {
            var response = new SalesOrderResponse();
            if (string.IsNullOrEmpty(transactionId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("transactionId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.SalesOrder = this.Controller.GetPendingSalesOrder(transactionId, includeSalesLines, this.SearchEngine);
            });
            
            return response;
        }
        
        #endregion

        /// <summary>
        /// Gets the sales orders by customer.
        /// </summary>
        /// <param name="includeSalesLines">if set to <c>true</c> [include sales lines].</param>
        /// <returns>
        /// The sales order collection.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override SalesOrderCollectionResponse GetSalesOrdersByCustomer(bool includeSalesLines)
        {
            return base.GetSalesOrdersByCustomer(includeSalesLines);
        }

        /// <summary>
        /// Gets the committed sales order.
        /// </summary>
        /// <param name="salesId">The sales identifier.</param>
        /// <param name="includeSalesLines">if set to <c>true</c> [include sales lines].</param>
        /// <returns>
        /// The sales order.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override SalesOrderResponse GetCommittedSalesOrder(string salesId, bool includeSalesLines)
        {
            var response = new SalesOrderResponse();
            if (string.IsNullOrEmpty(salesId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("salesId")));
            }

            return response.HasErrors ? response : base.GetCommittedSalesOrder(salesId, includeSalesLines);
        }

        /// <summary>
        /// Gets the committed sales order by confirmation identifier.
        /// </summary>
        /// <param name="confirmationId">The confirmation identifier.</param>
        /// <param name="includeSalesLines">if set to <c>true</c> [include sales lines].</param>
        /// <returns>
        /// The sales order.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override SalesOrderResponse GetCommittedSalesOrderByConfirmationId(string confirmationId, bool includeSalesLines)
        {
            var response = new SalesOrderResponse();
            if (string.IsNullOrEmpty(confirmationId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("confirmationId")));
            }

            return response.HasErrors ? response : base.GetCommittedSalesOrderByConfirmationId(confirmationId, includeSalesLines);
        }

        /// <summary>
        /// Gets the pending sales order.
        /// </summary>
        /// <param name="transactionId">The transaction identifier.</param>
        /// <param name="includeSalesLines">if set to <c>true</c> [include sales lines].</param>
        /// <returns>
        /// The sales order.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override SalesOrderResponse GetPendingSalesOrder(string transactionId, bool includeSalesLines)
        {
            var response = new SalesOrderResponse();
            if (string.IsNullOrEmpty(transactionId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("transactionId")));
            }

            return response.HasErrors ? response : base.GetPendingSalesOrder(transactionId, includeSalesLines);
        }
        
        /// <summary>
        /// Executes the given service action and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteAfterRequestValidation(ServiceResponse response, Action action)
        {
            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            var serviceUtility = ServiceUtility as ServiceUtility;
            if (serviceUtility != null)
            {
                serviceUtility.SetSslCertPolicy();
            }

            Utilities.ExecuteAndHandleExceptions(response, action);
        }

        /// <summary>
        /// Executes handler and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Action to execute.</param>
        protected override void Execute(ServiceResponse response, Action action)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Checks if user is authenticated and assosiated then calls <see cref="ExecuteAfterRequestValidation"/> with new execution handler.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteForAuthorizedUser(ServiceResponse response, Action<string> action)
        {
            string customerId = Utilities.GetCustomerIdFromCookie();

            ExecuteAfterRequestValidation(response, () => action(customerId));
        }        
    }
}
