﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="ShoppingCartService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2016
// </copyright>
// <summary>
//   The shopping cart dynamics service.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using Microsoft.Dynamics.Retail.Diagnostics;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.ServiceModel.Activation;
    using System.ServiceModel.Web;
    using System.Diagnostics.CodeAnalysis;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Controllers;

    /// <summary>
    /// Service for shopping cart.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class ShoppingCartService : ShoppingCartServiceBase, IExtendedShoppingCartService
    {
        private static readonly IProductValidator productValidator = new ProductValidator();
        private static readonly ISearchEngine searchEngine = new SearchEngine();
        private static readonly ServiceUtilityBase serviceProxy = new ServiceUtility();

        /// <summary>
        /// Gets Demo implementation of ISearchEngine.
        /// </summary>
        protected override ISearchEngine SearchEngine
        {
            get { return searchEngine; }
        }

        /// <summary>
        /// Gets Demo implementation of IProductValidator.
        /// </summary>
        protected override IProductValidator ProductValidator
        {
            get { return productValidator; }
        }

        /// <summary>
        /// Gets Demo implementation of ServiceUtilityBase.
        /// </summary>
        protected override ServiceUtilityBase ServiceUtility
        {
            get { return serviceProxy; }
        }

        /// <summary>
        /// Gets Demo implementation of IConfiguration.
        /// </summary>
        protected override IConfiguration Configuration
        {
            get { return new ServiceConfiguration(); }
        }

        /// <summary>
        /// Executes the given service action and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteAfterRequestValidation(ServiceResponse response, Action action)
        {
            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            var serviceUtility = ServiceUtility as ServiceUtility;
            if (serviceUtility != null)
            {
                serviceUtility.SetSslCertPolicy();
            }

            Utilities.ExecuteAndHandleExceptions(response, action);
        }

        /// <summary>
        /// Executes handler and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Action to execute.</param>
        protected override void Execute(ServiceResponse response, Action action)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Checks if user is authenticated and assosiated then calls <see>
        ///         <cref>ExecuteAfterPostRequestValidation</cref>
        ///     </see>
        ///     with new execution handler.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Action to execute.</param>
        protected override void ExecuteForAuthorizedUser(ServiceResponse response, Action<string> action)
        {
            throw new NotImplementedException();
        }

        #region EXTENSIONS

        /// <summary>
        /// Creates the controller.
        /// </summary>
        /// <returns>This controller</returns>
        protected override ShoppingCartController CreateController()
        {
            return Activator.CreateInstance<ExtendedShoppingCartController>();
        }
        
        /// <summary>
        /// Loads a shopping cart.
        /// </summary>
        /// <param name="shoppingCartId">The identifier for the shopping cart.</param>
        /// <param name="customerId">The identifier for the customer.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The service response containing the updated shopping cart.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse GetShoppingCartExtended(string shoppingCartId, string customerId, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(shoppingCartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("shoppingCartId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            ExecuteAfterRequestValidation(response, () =>
            {
                ShoppingCart shoppingCart = string.IsNullOrWhiteSpace(customerId)
                    ? this.Controller.GetShoppingCart(shoppingCartId, customerId, dataLevel, this.ProductValidator, this.SearchEngine)
                    : this.Controller.GetActiveShoppingCart(customerId, dataLevel, this.ProductValidator, this.SearchEngine);

                response.ShoppingCart = shoppingCart;
            });

            return response;
        }

        /// <summary>
        /// Adds items to the shopping cart associated with the given identifier.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="listings">The items to add to the cart.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The service response containing the updated shopping cart.</returns>
        [WebInvoke(Method = "POST", ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse AddItemsExtended(string shoppingCartId, string customerId, IEnumerable<Listing> listings, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(shoppingCartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("shoppingCartId")));
            }

            var products = listings as Listing[] ?? listings.ToArray();
            if (listings == null || !products.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listings")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = this.Controller.AddItems(shoppingCartId, customerId, products, dataLevel, this.ProductValidator, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Removes items from the shopping cart associated with the given identifier.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="lineIds">The item line identifiers to be removed from the shopping cart.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>
        /// The service response containing the updated shopping cart.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse RemoveItemsExtended(string shoppingCartId, string customerId, IEnumerable<string> lineIds, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(shoppingCartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("shoppingCartId")));
            }

            var ids = lineIds as string[] ?? lineIds.ToArray();
            if (lineIds == null || !ids.ToList().Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("lineIds")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = this.Controller.RemoveItems(shoppingCartId, customerId, ids, dataLevel, this.ProductValidator, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Updates items on the shopping cart associated with the given identifier.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="items">The items to be updated.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>
        /// The service response containing the updated shopping cart.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse UpdateItemsExtended(string shoppingCartId, string customerId, IEnumerable<TransactionItem> items, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(shoppingCartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("shoppingCartId")));
            }

            var transactionItems = items as TransactionItem[] ?? items.ToArray();
            if (items == null || !transactionItems.ToList().Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("items")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = this.Controller.UpdateItems(shoppingCartId, customerId, transactionItems, dataLevel, this.ProductValidator, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Merges anonymous and active shopping carts for the current user.
        /// </summary>
        /// <param name="anonymousShoppingCartId">The anonymous shopping cart id</param>
        /// <param name="customerId">The customer id</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The service response containing the merged shopping cart.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse MergeCarts(string anonymousShoppingCartId, string customerId, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(anonymousShoppingCartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("anonymousShoppingCartId")));
            }

            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            ExecuteAfterRequestValidation(response, () =>
            {
                ShoppingCart anonymousShoppingCart = this.Controller.GetShoppingCart(anonymousShoppingCartId, string.Empty, dataLevel, this.ProductValidator, this.SearchEngine);
                if (anonymousShoppingCart != null && anonymousShoppingCart.Items.Any())
                {
                    ShoppingCart authenticatedShoppingCart = this.Controller.GetActiveShoppingCart(customerId, dataLevel, this.ProductValidator, this.SearchEngine);
                    response.ShoppingCart = authenticatedShoppingCart == null 
                        ? this.Controller.ClaimAnonymousCart(anonymousShoppingCartId, customerId, dataLevel, this.ProductValidator, this.SearchEngine) 
                        : this.Controller.MoveItemsBetweenCarts(anonymousShoppingCartId, authenticatedShoppingCart.CartId, customerId, dataLevel, this.ProductValidator, this.SearchEngine);
                    
                    // REMOVE ITEM FROM ANOMYMOUS USER'S CART
                    try
                    {
                        this.Controller.RemoveItems(anonymousShoppingCartId, string.Empty, anonymousShoppingCart.Items.ToList().Select(i => i.LineId), ShoppingCartDataLevel.Minimal, this.ProductValidator, this.SearchEngine);
                    }
                    catch (Exception e)
                    {
                        NetTracer.Error(e, "Failed to delete shopping cart after shopping merge.", new object[0]);
                    }
                }
            });

            return response;
        }

        /// <summary>
        /// Gets all the promotions for the shopping cart items.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>
        /// The service response containing the updated shopping cart.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse GetPromotionsExtended(string shoppingCartId, string customerId, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(shoppingCartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("shoppingCartId")));
            }
            
            if (response.HasErrors)
            {
                return response;
            }

            ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = this.Controller.GetAllPromotionsForShoppingCart(shoppingCartId, dataLevel, this.ProductValidator, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Adds or Removes the discount codes from the cart.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="promotionCode">The promotion code to apply to the cart.</param>
        /// <param name="isAdd">Indicates whether the operation is addition or removal of discount codes.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The shopping cart.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ShoppingCartResponse AddOrRemovePromotionCodeExtended(string shoppingCartId, string customerId, string promotionCode, bool isAdd, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(shoppingCartId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("shoppingCartId")));
            }

            if (string.IsNullOrEmpty(promotionCode))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("promotionCode")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            ExecuteAfterRequestValidation(response, () =>
            {
                response.ShoppingCart = this.Controller.AddOrRemovePromotionCode(shoppingCartId, customerId, promotionCode, isAdd, dataLevel, this.ProductValidator, this.SearchEngine);
            });

            return response;
        }

        #endregion

        /// <summary>
        /// Adds items to the shopping cart associated with the given identifier.
        /// </summary>
        /// <param name="isCheckoutSession">Indicates whether this is in context of a checkout session.</param>
        /// <param name="listings">The items to add to the cart.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>
        /// The service response containing the updated shopping cart.
        /// </returns>
        [WebInvoke(Method = "POST", ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse AddItems(bool isCheckoutSession, IEnumerable<Listing> listings, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            var products = listings as Listing[] ?? listings.ToArray();
            if (listings == null || !products.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listings")));
            }

            return response.HasErrors ? response : base.AddItems(isCheckoutSession, products, dataLevel);
        }

        /// <summary>
        /// Gets the shopping cart associated .
        /// </summary>
        /// <param name="isCheckoutSession">Indicates whether the request originated from the checkout page.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>
        /// The service response containing the shopping cart.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse GetShoppingCart(bool isCheckoutSession, ShoppingCartDataLevel dataLevel)
        {
            return base.GetShoppingCart(isCheckoutSession, dataLevel);
        }

        /// <summary>
        /// Removes items from the shopping cart associated with the given identifier.
        /// </summary>
        /// <param name="isCheckoutSession">Indicates whether this is in context of a checkout session.</param>
        /// <param name="lineIds">The item line identifiers to be removed from the shopping cart.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>
        /// The service response containing the updated shopping cart.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse RemoveItems(bool isCheckoutSession, IEnumerable<string> lineIds, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            var ids = lineIds as string[] ?? lineIds.ToArray();
            if (lineIds == null || !ids.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("lineIds")));
            }

            return response.HasErrors ? response : base.RemoveItems(isCheckoutSession, ids, dataLevel);
        }

        /// <summary>
        /// Updates items on the shopping cart associated with the given identifier.
        /// </summary>
        /// <param name="isCheckoutSession">Indicates whether this is in context of a checkout session.</param>
        /// <param name="items">The items to be updated.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>
        /// The service response containing the updated shopping cart.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse UpdateItems(bool isCheckoutSession, IEnumerable<TransactionItem> items, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            var transactionItems = items as TransactionItem[] ?? items.ToArray();
            if (items == null || !transactionItems.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("items")));
            }

            return response.HasErrors ? response : base.UpdateItems(isCheckoutSession, transactionItems, dataLevel);
        }

        /// <summary>
        /// Gets all the promotions for the shopping cart items.
        /// </summary>
        /// <param name="isCheckoutSession">Indicates whether this is in context of a checkout session.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>
        /// The service response containing the updated shopping cart.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override ShoppingCartResponse GetPromotions(bool isCheckoutSession, ShoppingCartDataLevel dataLevel)
        {
            var response = base.GetPromotions(isCheckoutSession, dataLevel);
            Utilities.LoadProductDetails(response.ShoppingCart);
            return response;
        }

        /// <summary>
        /// Adds or Removes the discount codes from the cart.
        /// </summary>
        /// <param name="isCheckoutSession">Indicates whether this is in context of a checkout session.</param>
        /// <param name="promotionCode">The promotion code to apply to the cart.</param>
        /// <param name="isAdd">Indicates whether the operation is addition or removal of discount codes.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The shopping cart.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse AddOrRemovePromotionCode(bool isCheckoutSession, string promotionCode, bool isAdd, ShoppingCartDataLevel dataLevel)
        {
            var response = new ShoppingCartResponse();
            if (string.IsNullOrEmpty(promotionCode))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("promotionCode")));
            }

            return response.HasErrors ? response : base.AddOrRemovePromotionCode(isCheckoutSession, promotionCode, isAdd, dataLevel);
        }

        /// <summary>
        /// Gets information about the specific kit configuration that has the provided kit line values.
        /// </summary>
        /// <param name="kitProductMasterIdentifier">The product identifier of the kit product master.</param>
        /// <param name="catalogId">The catalog identifier.</param>
        /// <param name="kitLines">Details of each component line that comprises the kit.</param>
        /// <returns>Specific kit configuration information.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override KitConfigurationResponse GetKitConfigurationInformation(long kitProductMasterIdentifier, long catalogId, IEnumerable<KitLine> kitLines)
        {
            return base.GetKitConfigurationInformation(kitProductMasterIdentifier, catalogId, kitLines);
        }

        /// <summary>
        /// Gets the applicable variants of a kit component.
        /// </summary>
        /// <param name="kitComponentProductId">Product identifier of the kit component.</param>
        /// <param name="kitComponentParentId">Product identifier of the kit component master.</param>
        /// <param name="parentKitId">Product identifier of the kit master product.</param>
        /// <param name="kitComponentLineId">The kit component line identifier.</param>
        /// <param name="catalogId">The catalog identifier.</param>
        /// <returns>Returns a collection of kit component variants.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override KitComponentVariantResponse GetKitComponentVariants(long kitComponentProductId, long kitComponentParentId, long parentKitId, long kitComponentLineId, long catalogId)
        {
            return base.GetKitComponentVariants(kitComponentProductId, kitComponentParentId, parentKitId, kitComponentLineId, catalogId);
        }

        /// <summary>
        /// Starts the checkout process by creating a secure cart and returning it.
        /// </summary>
        /// <param name="dataLevel">The shopping cart data level.</param>
        /// <returns>A new cart with a random cart id that should be used during the secure checkout process.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ShoppingCartResponse CommenceCheckout(ShoppingCartDataLevel dataLevel)
        {
            var response = base.CommenceCheckout(dataLevel);
            Utilities.LoadProductDetails(response.ShoppingCart);
            return response;
        }
    }
}

