﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="StoreProductAvailabilityService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2016
// </copyright>
// <summary>
//   The store product availability dynamics service.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System;
    using System.Collections.Generic;
    using System.ServiceModel.Activation;
    using System.ServiceModel.Web;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using System.Linq;

    /// <summary>
    /// Service for store availability.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class StoreProductAvailabilityService : StoreProductAvailabilityServiceBase
    {
        private static readonly IProductValidator productValidator = new ProductValidator();
        private static readonly ISearchEngine searchEngine = new SearchEngine();
        private static readonly ServiceUtilityBase serviceProxy = new ServiceUtility();
        
        /// <summary>
        /// Gets Demo implementation of ISearchEngine.
        /// </summary>
        protected override ISearchEngine SearchEngine
        {
            get { return searchEngine; }
        }

        /// <summary>
        /// Gets Demo implementation of IProductValidator.
        /// </summary>
        protected override IProductValidator ProductValidator
        {
            get { return productValidator; }
        }

        /// <summary>
        /// Gets Demo implementation of ServiceUtilityBase.
        /// </summary>
        protected override ServiceUtilityBase ServiceUtility
        {
            get { return serviceProxy; }
        }

        /// <summary>
        /// Gets Demo implementation of IConfiguration.
        /// </summary>
        protected override IConfiguration Configuration
        {
            get { return new ServiceConfiguration(); }
        }

        /// <summary>
        /// Gets the nearby stores with availability.
        /// </summary>
        /// <param name="latitude">The latitude.</param>
        /// <param name="longitude">The longitude.</param>
        /// <param name="items">The cart items.</param>
        /// <returns>
        /// Stores with product availability.
        /// </returns>
        [WebInvoke(Method = "POST", ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override StoreProductAvailabilityResponse GetNearbyStoresWithAvailability(decimal latitude, decimal longitude, IEnumerable<TransactionItem> items)
        {
            var response = new StoreProductAvailabilityResponse();
            if (latitude == 0M)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("latitude")));
            }

            if (longitude == 0M)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("longitude")));
            }

            var transactionItems = items as TransactionItem[] ?? items.ToArray();
            if (items == null || !transactionItems.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("items")));
            }

            return response.HasErrors ? response : base.GetNearbyStoresWithAvailability(latitude, longitude, transactionItems);
        }

        /// <summary>
        /// Gets the quantities available of the specified listings.
        /// </summary>
        /// <param name="listingIds">Listing identifiers.</param>
        /// <returns>Available quantities for the listings inquired.</returns>
        [WebInvoke(Method = "POST", ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override ListingAvailableQuantityResponse GetListingAvailableQuantities(IEnumerable<long> listingIds)
        {
            var response = new ListingAvailableQuantityResponse();
            var productIds = listingIds as long[] ?? listingIds.ToArray();
            if (listingIds == null || !productIds.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingIds")));
            }

            return response.HasErrors ? response : base.GetListingAvailableQuantities(productIds);
        }

        /// <summary>
        /// Gets the nearby stores.
        /// </summary>
        /// <param name="latitude">The latitude.</param>
        /// <param name="longitude">The longitude.</param>
        /// <param name="distance">miles to search in</param>
        /// <returns>
        /// Stores.
        /// </returns>
        [WebInvoke(Method = "POST", ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override StoreLocationResponse GetNearbyStores(decimal latitude, decimal longitude, int distance)
        {
            var response = new StoreLocationResponse();
            if (latitude == 0M)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("latitude")));
            }

            if (longitude == 0M)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("longitude")));
            }

            return response.HasErrors ? response : base.GetNearbyStores(latitude, longitude, distance);
        }
        
        /// <summary>
        /// Executes the given service action and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteAfterRequestValidation(ServiceResponse response, System.Action action)
        {
            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            var serviceUtility = ServiceUtility as ServiceUtility;
            if (serviceUtility != null)
            {
                serviceUtility.SetSslCertPolicy();
            }

            Utilities.ExecuteAndHandleExceptions(response, action);
        }

        /// <summary>
        /// Executes handler and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void Execute(ServiceResponse response, Action action)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Checks if user is authenticated and assosiated then calls <see cref="ExecuteForAuthorizedUser" /> with new execution handler.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void ExecuteForAuthorizedUser(ServiceResponse response, Action<string> action)
        {
            throw new NotImplementedException();
        }
    }
}
