﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="WishListService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2016
// </copyright>
// <summary>
//   The wish list dynamics service.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System.ServiceModel.Activation;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using System;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using System.Collections.Generic;
    using System.ServiceModel.Web;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using System.Diagnostics.CodeAnalysis;
    using System.Linq;

    /// <summary>
    /// The wish list dynamics service.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class WishListService : WishListServiceBase, IExtendedWishListService
    {
        private static readonly IProductValidator productValidator = new ProductValidator();
        private static readonly ISearchEngine searchEngine = new SearchEngine();
        private static readonly ServiceUtilityBase serviceProxy = new ServiceUtility();

        /// <summary>
        /// Gets the search engine.
        /// </summary>
        /// <value>
        /// The search engine.
        /// </value>
        protected override ISearchEngine SearchEngine
        {
            get { return searchEngine; }
        }

        /// <summary>
        /// Gets the product validator.
        /// </summary>
        /// <value>
        /// The product validator.
        /// </value>
        protected override IProductValidator ProductValidator
        {
            get { return productValidator; }
        }

        /// <summary>
        /// Gets the service utility.
        /// </summary>
        /// <value>
        /// The service utility.
        /// </value>
        protected override ServiceUtilityBase ServiceUtility
        {
            get { return serviceProxy; }
        }

        /// <summary>
        /// Gets the configuration.
        /// </summary>
        /// <value>
        /// The configuration.
        /// </value>
        protected override IConfiguration Configuration
        {
            get { return new ServiceConfiguration(); }
        }

        /// <summary>
        /// Executes the specified response.
        /// </summary>
        /// <param name="response">The response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void Execute(ServiceResponse response, Action action)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Executes the after request validation.
        /// </summary>
        /// <param name="response">The response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.ArgumentNullException">action</exception>
        protected override void ExecuteAfterRequestValidation(ServiceResponse response, Action action)
        {
            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            Utilities.ExecuteAndHandleExceptions(response, action);
        }

        /// <summary>
        /// Executes for authorized user.
        /// </summary>
        /// <param name="response">The response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void ExecuteForAuthorizedUser(ServiceResponse response, Action<string> action)
        {
            throw new NotImplementedException();
        }

        #region EXTENSIONS

        /// <summary>
        /// Adds the items to wish list.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <param name="listings">The listings.</param>
        /// <returns>
        /// A wish list response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual WishListResponse AddItemsToWishListExtended(string customerId, string wishListId, IEnumerable<Listing> listings)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            var products = listings as Listing[] ?? listings.ToArray();
            if (listings == null || !products.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listings")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.WishList = this.Controller.AddItemsToWishList(wishListId, customerId, products);
            });

            return response;
        }

        /// <summary>
        /// Creates the wish list.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="wishListName">Name of the wish list.</param>
        /// <returns>
        /// A wish list response
        /// </returns> 
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual WishListResponse CreateWishListExtended(string customerId, string wishListName)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            var wishList = new WishList { Name = wishListName };
            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.WishList = this.Controller.CreateWishList(wishList, customerId);
            });

            return response;
        }

        /// <summary>
        /// Gets the wish list.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <returns>
        /// A wish list response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual WishListResponse GetWishListExtended(string customerId, string wishListId)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.WishList = this.Controller.GetWishList(wishListId, customerId, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Gets the wish lists for current customer.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <returns>
        /// A wish list collection response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual WishListCollectionResponse GetWishListsForCurrentCustomerExtended(string customerId)
        {
            var response = new WishListCollectionResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (response.HasErrors)
            {
                return response;
            }
            
            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.WishLists = this.Controller.GetWishListsForCustomer(customerId, this.SearchEngine);
            });

            return response;
        }

        /// <summary>
        /// Removes the item from wish list.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <param name="listingId">The listing identifier.</param>
        /// <returns>
        /// A null response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual NullResponse RemoveItemFromWishListExtended(string customerId, string wishListId, string listingId)
        {
            var response = new NullResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            if (string.IsNullOrEmpty(listingId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () => this.Controller.RemoveItemFromWishList(wishListId, customerId, listingId));

            return response;
        }

        /// <summary>
        /// Updates the items on wish list.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <param name="listings">The listings.</param>
        /// <returns>
        /// A wish list response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual WishListResponse UpdateItemsOnWishListExtended(string customerId, string wishListId, IEnumerable<Listing> listings)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            var products = listings as Listing[] ?? listings.ToArray();
            if (listings == null || !products.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listings")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.WishList = this.Controller.UpdateItemsOnWishList(wishListId, customerId, products);
            });

            return response;
        }

        /// <summary>
        /// Updates the wish list properties.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <param name="wishListName">Name of the wish list.</param>
        /// <param name="isFavorite">The is favorite.</param>
        /// <returns>
        /// A wish list response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual WishListResponse UpdateWishListPropertiesExtended(string customerId, string wishListId, string wishListName, bool? isFavorite)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            var wishList = new WishList
            {
                Id = wishListId,
                Name = wishListName,
                IsFavorite = isFavorite,
                CustomerId = customerId
            };

            this.ExecuteAfterRequestValidation(response, delegate
            {
                response.WishList = this.Controller.UpdateWishListProperties(wishList, customerId);
            });

            return response;
        }

        /// <summary>
        /// Deletes the wish list.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <returns>
        /// A wish list response
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual NullResponse DeleteWishListExtended(string customerId, string wishListId)
        {
            var response = new NullResponse();
            if (string.IsNullOrEmpty(customerId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customerId")));
            }

            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () => this.Controller.DeleteWishList(wishListId, customerId));

            return response;
        }

        #endregion

        /// <summary>
        /// Adds the items to wish list.
        /// </summary>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <param name="listings">The listings.</param>
        /// <returns>A wish list response</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override WishListResponse AddItemsToWishList(string wishListId, IEnumerable<Listing> listings)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            var products = listings as Listing[] ?? listings.ToArray();
            if (listings == null || !products.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listings")));
            }

            return response.HasErrors ? response : base.AddItemsToWishList(wishListId, products);
        }

        /// <summary>
        /// Creates the wish list.
        /// </summary>
        /// <param name="wishListName">Name of the wish list.</param>
        /// <returns>A wish list response</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override WishListResponse CreateWishList(string wishListName)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(wishListName))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListName")));
            }

            return response.HasErrors ? response : base.CreateWishList(wishListName);
        }

        /// <summary>
        /// Deletes the wish list.
        /// </summary>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <returns>A wish list response</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override NullResponse DeleteWishList(string wishListId)
        {
            var response = new NullResponse();
            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            return response.HasErrors ? response : base.DeleteWishList(wishListId);
        }

        /// <summary>
        /// Gets the wish list.
        /// </summary>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <returns>A wish list response</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override WishListResponse GetWishList(string wishListId)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            return response.HasErrors ? response : base.GetWishList(wishListId);
        }

        /// <summary>
        /// Gets the wish lists for current customer.
        /// </summary>
        /// <returns>A wish list response</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services.WishListCollectionResponse GetWishListsForCurrentCustomer()
        {
            return base.GetWishListsForCurrentCustomer();
        }

        /// <summary>
        /// Removes the item from wish list.
        /// </summary>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <param name="listingId">The listing identifier.</param>
        /// <returns>A null response</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override NullResponse RemoveItemFromWishList(string wishListId, string listingId)
        {
            var response = new NullResponse();
            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            if (string.IsNullOrEmpty(listingId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingId")));
            }

            return response.HasErrors ? response : base.RemoveItemFromWishList(wishListId, listingId);
        }

        /// <summary>
        /// Updates the items on wish list.
        /// </summary>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <param name="listings">The listings.</param>
        /// <returns>A wish list response</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override WishListResponse UpdateItemsOnWishList(string wishListId, IEnumerable<Listing> listings)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }

            var products = listings as Listing[] ?? listings.ToArray();
            if (listings == null || !products.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listings")));
            }

            return response.HasErrors ? response : base.UpdateItemsOnWishList(wishListId, products);
        }

        /// <summary>
        /// Updates the wish list properties.
        /// </summary>
        /// <param name="wishListId">The wish list identifier.</param>
        /// <param name="wishListName">Name of the wish list.</param>
        /// <param name="isFavorite">The is favorite.</param>
        /// <returns>A wish list response</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override WishListResponse UpdateWishListProperties(string wishListId, string wishListName, bool? isFavorite)
        {
            var response = new WishListResponse();
            if (string.IsNullOrEmpty(wishListId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("wishListId")));
            }
            
            return response.HasErrors ? response : base.UpdateWishListProperties(wishListId, wishListName, isFavorite);
        }
    }
}
